module bandpass_mod

! Apply a bandpass filter on one axis of the data

  use sep

  implicit none

  include "fftw3.f"

  integer, private                                 :: n, nn, np, i, j, k, iw
  real, private                                    :: d, dw, w, pi
  integer*8, private                               :: planf, planb
  real, dimension(:), allocatable, private         :: wt
  complex, dimension(:), allocatable, private      :: c1, c2

  contains

!!---------------------------------------------------------- 
  ! Initialization: n  = n of axis to be transformed
  !                 np = number of samples to pad
  !                 d  = d of axis to be transformed
  !                 w1 = beginning of ramp up taper
  !                 w2 = end of ramp up taper
  !                 w3 = beginning of ramp down taper
  !                 w4 = end of ramp down taper

  subroutine bandpass_init(n_in, np_in, d_in, w1, w2, w3, w4)
    integer                       :: n_in, np_in
    real                          :: d_in, w1, w2, w3, w4

    n = n_in
    np = np_in
    d = d_in

    if(w2 .eq. w1 .or. w4 .eq. w3) call seperr("w1 equal to w2 or w3 equal to w4")

    if(np < 0) call erexit("np is negative")
    if(w2 < w1 .or. w3 < w2 .or. w4 < w3) write(0,*) "WARNING: w1 w2 w3 w4 are not in ascending order"
    if(w1 < 0. .or. w1 > 1./(2.*d)) write(0,*) "WARNING: w1 is outside the frequency range of the data"
    if(w2 < 0. .or. w2 > 1./(2.*d)) write(0,*) "WARNING: w2 is outside the frequency range of the data"
    if(w3 < 0. .or. w3 > 1./(2.*d)) write(0,*) "WARNING: w3 is outside the frequency range of the data"
    if(w4 < 0. .or. w4 > 1./(2.*d)) write(0,*) "WARNING: w4 is outside the frequency range of the data"

    nn = n + np
    dw = 1./(nn*d)
    pi = 3.14159265358979323846

    if(allocated(c1)) deallocate(c1)
    if(allocated(c2)) deallocate(c2)
    if(allocated(wt)) deallocate(wt)
    allocate(c1(nn))
    allocate(c2(nn))
    allocate(wt(nn))

    wt = 1.
    do iw=1,nn
      w = (iw-1)*dw
      if(w < w1) then
        wt(iw) = 0.
      elseif(w >= w1 .and. w <= w2) then
        wt(iw) = wt(iw)*cos(pi/2.*(w2-w)/(w2-w1))*cos(pi/2.*(w2-w)/(w2-w1))
      elseif(w >= w3 .and. w <= w4) then
        wt(iw) = wt(iw)*cos(pi/2.*(w-w3)/(w4-w3))*cos(pi/2.*(w-w3)/(w4-w3))
      elseif(w > w4) then
        wt(iw) = 0.
      endif
    enddo

!call snap('wt.H',nn,1,wt)

    call sfftw_plan_dft_1d(planf, nn, c1, c2, FFTW_FORWARD, FFTW_MEASURE)
    call sfftw_plan_dft_1d(planb, nn, c2, c1, FFTW_BACKWARD, FFTW_MEASURE)

  end subroutine

!!---------------------------------------------------------- 
! bandpass 1D data

  subroutine bandpass_1d(data)
    real, dimension(:)            :: data

    if(size(data,1) .ne. n) call erexit("size of input data does not match initialization parameter")

    c1 = 0.
    c2 = 0.

    c1(1:n) = data
    call sfftw_execute_dft(planf, c1, c2)

    c2 = c2 * wt

    c1 = 0.
    call sfftw_execute_dft(planb, c2, c1)

    data = 2.*real(c1(1:n))/nn

  end subroutine

!!---------------------------------------------------------- 
! bandpass first axis of 2D data

  subroutine bandpass_2d1(data)
    real, dimension(:,:)           :: data

    if(size(data,1) .ne. n) call erexit("size of input data does not match initialization parameter")

    do j=1,size(data,2)
      c1 = 0.
      c2 = 0.

      c1(1:n) = data(:,j)
      call sfftw_execute_dft(planf, c1, c2)

      c2 = c2 * wt

      c1 = 0.
      call sfftw_execute_dft(planb, c2, c1)

      data(:,j) = 2.*real(c1(1:n))/nn
    enddo

  end subroutine

!!---------------------------------------------------------- 
! bandpass second axis of 2D data

  subroutine bandpass_2d2(data)
    real, dimension(:,:)           :: data

    if(size(data,2) .ne. n) call erexit("size of input data does not match initialization parameter")

    do i=1,size(data,1)
      c1 = 0.
      c2 = 0.

      c1(1:n) = data(i,:)
      call sfftw_execute_dft(planf, c1, c2)

      c2 = c2 * wt

      c1 = 0.
      call sfftw_execute_dft(planb, c2, c1)

      data(i,:) = 2.*real(c1(1:n))/nn
    enddo

  end subroutine

!!---------------------------------------------------------- 
! bandpass first axis of 3D data

  subroutine bandpass_3d1(data)
    real, dimension(:,:,:)           :: data

    if(size(data,1) .ne. n) call erexit("size of input data does not match initialization parameter")

    do k=1,size(data,3)
      do j=1,size(data,2)
        c1 = 0.
        c2 = 0.

        c1(1:n) = data(:,j,k)
        call sfftw_execute_dft(planf, c1, c2)

        c2 = c2 * wt

        c1 = 0.
        call sfftw_execute_dft(planb, c2, c1)

        data(:,j,k) = 2.*real(c1(1:n))/nn
      enddo
    enddo

  end subroutine

!!---------------------------------------------------------- 
! bandpass second axis of 3D data

  subroutine bandpass_3d2(data)
    real, dimension(:,:,:)           :: data

    if(size(data,2) .ne. n) call erexit("size of input data does not match initialization parameter")

    do k=1,size(data,3)
      do i=1,size(data,1)
        c1 = 0.
        c2 = 0.

        c1(1:n) = data(i,:,k)
        call sfftw_execute_dft(planf, c1, c2)

        c2 = c2 * wt

        c1 = 0.
        call sfftw_execute_dft(planb, c2, c1)

        data(i,:,k) = 2.*real(c1(1:n))/nn
      enddo
    enddo

  end subroutine

!!---------------------------------------------------------- 
! bandpass third axis of 3D data

  subroutine bandpass_3d3(data)
    real, dimension(:,:,:)           :: data

    if(size(data,3) .ne. n) call erexit("size of input data does not match initialization parameter")

    do j=1,size(data,2)
      do i=1,size(data,1)
        c1 = 0.
        c2 = 0.

        c1(1:n) = cmplx(data(i,j,:),0.)
        call sfftw_execute_dft(planf, c1, c2)

        c2 = c2 * wt

        c1 = 0.
        call sfftw_execute_dft(planb, c2, c1)

        data(i,j,:) = 2.*real(c1(1:n))/nn
      enddo
    enddo

  end subroutine

!!---------------------------------------------------------- 

end module
